AddCSLuaFile("cl_init.lua")
AddCSLuaFile("shared.lua")

include("shared.lua")

function ENT:Initialize()
	self:SetModel("models/crossbow_bolt.mdl")

	if self.HealthOverride then
		self:SetMaxNailHealth(self.HealthOverride)
		self:SetNailHealth(self.HealthOverride)
	else
		local health = GAMEMODE.NailHealth
		if self.HealthMultiplier then health = health * self.HealthMultiplier end
		self:SetMaxNailHealth(health)
		self:SetNailHealth(health)
	end

	self:SetRepairMultiplier(1)

	self.m_NextStrainSound = 0
end

function ENT:OnDamaged(damage, attacker, inflictor, dmginfo)
	if CurTime() >= self.m_NextStrainSound then
		self.m_NextStrainSound = CurTime() + math.min(damage * 0.025, 1)

		self:EmitSound("physics/metal/metal_box_impact_hard"..math.random(1, 3)..".wav", math.Clamp(damage * 2.5, 60, 80), math.min(255, 150 + (1 - (self:GetNailHealth() / self:GetMaxNailHealth())) * 100))
	end
end

function ENT:SetDeployer(pl)
	if not pl then return end

	if type(pl) == "string" then
		self:SetNetworkedString("deployer", "")
		self:SetNetworkedString("deployer2", pl)
		self.OwnerUID = nil
	elseif pl:IsValid() then
		self:SetNetworkedString("deployer", pl:UniqueID())
		self:SetNetworkedString("deployer2", "")
		self.OwnerUID = pl:UniqueID()
	else
		self:SetNetworkedString("deployer", "")
		self:SetNetworkedString("deployer2", "")
		self.OwnerUID = nil
	end
end

function ENT:GetDeployer()
	local uid = self.OwnerUID
	if uid then
		for _, pl in pairs(player.GetAll()) do
			if pl:UniqueID() == uid then return pl end
		end
	end

	return NULL
end

function ENT:AttachTo(baseent, attachent, physbone, physbone2)
	self:SetBaseEntity(baseent)
	self:SetAttachEntity(attachent, physbone, physbone2)

	if not baseent.Nails then baseent.Nails = {} end
	if not attachent.Nails then attachent.Nails = {} end

	table.insert(baseent.Nails, self)
	table.insert(attachent.Nails, self)

	self:SetParentPhysNum(physbone or 0)
	self:SetParent(baseent)

	if baseent:IsValid() then
		baseent:CollisionRulesChanged()
	end
	if attachent:IsValid() then
		attachent:CollisionRulesChanged()
	end
end

function ENT:SetNailConstraint(const)
	self.m_Constraint = const
end

function ENT:GetNailConstraint()
	return self.m_Constraint or NULL
end

function ENT:AcceptInput(name, activator, caller, args)
	name = string.lower(name)
	if name == "sethealth" then
		self:SetNailHealth(tonumber(args) or 1)

		return true
	elseif name == "attachto" then
		local ent = ents.FindByName(args)[1]
		if ent and ent:IsValid() then
			self:SetParent(ent)
		end

		return true
	elseif name == "nailto" then
		if self:GetParent():IsValid() then
			local ent = args == "worldspawn" and GetWorldEntity() or ents.FindByName(args)[1]
			if ent then
				self:AttachTo(self:GetParent(), ent)
			end
		end

		return true
	elseif name == "setname" or name == "setdeployer" then
		self:SetDeployer(args)

		return true
	elseif name == "setunremoveable" or name == "setunremovable" then
		self.m_NailUnremovable = tonumber(args) == 1

		return true
	elseif name == "toggleunremoveable" or name == "toggleunremovable" then
		self.m_NailUnremovable = not self.m_NailRemovable

		return true
	end
end

function ENT:OnRemove()
	if not self.m_IsRemoving then
		local baseent = self:GetBaseEntity()
		if baseent:IsValid() and not baseent:IsWorld() then
			baseent:RemoveNail(self)
		end
		local attachent = self:GetAttachEntity()
		if attachent:IsValid() and not attachent:IsWorld() then
			attachent:RemoveNail(self)
		end
	end
end

function ENT:KeyValue(key, value)
	key = string.lower(key)
	if key == "healthoverride" then
		self.HealthOverride = tonumber(value)
	elseif key == "healthmultiplier" or key == "healthmul" then
		self.HealthMultiplier = tonumber(value)
	elseif key == "unremoveable" or key == "unremovable" then
		self.m_NailUnremoveable = tonumber(value) == 1
	end
end
