AddCSLuaFile("shared.lua")
AddCSLuaFile("cl_init.lua")

include("shared.lua")

SWEP.PreviousNailHealth = {}
SWEP.PreviousNailRepairMultiplier = {}

function SWEP:Reload()
	if CurTime() < self:GetNextPrimaryFire() then return end

	local owner = self.Owner
	if owner:GetBarricadeGhosting() then return end

	local startpos = owner:GetShootPos()
	local ent
	local dist
	local tr = owner:MeleeTrace(self.MeleeRange, self.MeleeSize, owner:GetMeleeFilter())
	local trent = tr.Entity
	if not trent:IsValid() or not trent:IsNailed() then return end

	for _, e in pairs(ents.FindByClass("prop_nail")) do
		if not e.m_PryingOut and e:GetParent() == trent then
			local edist = e:GetActualPos():Distance(tr.HitPos)
			if edist <= 8 and (not dist or edist < dist) then
				ent = e
				dist = edist
			end
		end
	end

	if not ent then return end

	self:SetNextPrimaryFire(CurTime() + 1)

	ent.m_PryingOut = true -- Prevent loops.

	if self.Alternate then
		self:SendWeaponAnim(ACT_VM_HITCENTER)
	else
		self:SendWeaponAnim(ACT_VM_MISSCENTER)
	end
	self.Alternate = not self.Alternate

	owner:DoAnimationEvent(ACT_HL2MP_GESTURE_RANGE_ATTACK_MELEE)

	owner:EmitSound("weapons/melee/crowbar/crowbar_hit-"..math.random(1,4)..".wav")

	local nailowner = player.GetByUniqueID(ent.OwnerUID)
	ent:GetParent():RemoveNail(ent, nil, self.Owner)

	table.insert(self.PreviousNailHealth, ent:GetNailHealth())
	table.insert(self.PreviousNailRepairMultiplier, ent:GetRepairMultiplier())

	if nailowner and nailowner ~= owner and nailowner:Team() == TEAM_HUMAN then
		if not gamemode.Call("PlayerIsAdmin", owner) then
			owner:GivePenalty(30)
			owner:ReflectDamage(20)
		end

		if nailowner:NearestPoint(tr.HitPos):Distance(tr.HitPos) <= 768 and nailowner:HasWeapon("weapon_zs_hammer") then
			nailowner:GiveAmmo(1, self.Primary.Ammo)
		else
			owner:GiveAmmo(1, self.Primary.Ammo)
		end
	else
		owner:GiveAmmo(1, self.Primary.Ammo)
	end
end

function SWEP:OnMeleeHit(hitent, hitflesh, tr)
	if hitent:IsValid() then
		if hitent.HitByHammer and hitent:HitByHammer(self, self.Owner, tr) then
			return
		end

		if hitent:IsNailed() then
			local possiblenails = {}
			for _, nail in pairs(hitent.Nails) do
				if nail and nail:IsValid() and tr.HitPos:Distance(nail:GetActualPos()) <= 8 and (nail.m_NextRepairAllowed or 0) <= CurTime() then
					possiblenails[#possiblenails + 1] = nail
				end
			end

			if #possiblenails == 0 then return end

			local lowestnail
			local lowesthealth
			for _, nail in pairs(possiblenails) do
				if not lowesthealth or nail:GetNailHealth() < lowesthealth then
					lowesthealth = nail:GetNailHealth()
					lowestnail = nail
				end
			end

			if lowestnail and lowestnail:IsValid() and lowestnail:GetNailHealth() < lowestnail:GetMaxNailHealth() * 0.8 then
				local repairmultiplier = lowestnail:GetRepairMultiplier()
				local multiplier = repairmultiplier * (self.Owner.HumanRepairMultiplier or 1)
				if multiplier > 0 then
					local old = lowestnail:GetNailHealth()
					lowestnail:SetNailHealth(math.min(old + self.RepairRate * multiplier, lowestnail:GetMaxNailHealth()))
					lowestnail.m_NextRepairAllowed = CurTime() + 0.25

					local repaired = lowestnail:GetNailHealth() - old
					if repaired > 0 then
						lowestnail:SetRepairMultiplier(repairmultiplier - 0.015) -- A nail's repair rate decreases over time and eventually hits 0.
						self:PlayRepairSound(lowestnail)
						gamemode.Call("PlayerRepairedObject", self.Owner, lowestnail, repaired, self)
					end
				end

				local effectdata = EffectData()
					effectdata:SetOrigin(tr.HitPos)
					effectdata:SetNormal(tr.HitNormal)
					effectdata:SetMagnitude(multiplier)
				util.Effect("nailrepaired", effectdata, true, true)

				return true
			end
		end
	end
end

local NONAILS = {}
NONAILS[MAT_GRATE] = "Impossible."
NONAILS[MAT_CLIP] = "Impossible."
NONAILS[MAT_GLASS] = "Trying to put nails in glass is a silly thing to do."

function SWEP:SecondaryAttack()
	if self:GetPrimaryAmmoCount() <= 0 or CurTime() < self:GetNextPrimaryFire() or self.Owner:GetBarricadeGhosting() then return end

	local owner = self.Owner

	local tr = owner:TraceLine(64, MASK_SOLID, owner:GetMeleeFilter())
	local trent = tr.Entity

	if not trent:IsValid() or not util.IsValidPhysicsObject(trent, tr.PhysicsBone) or trent:GetMoveType() ~= MOVETYPE_VPHYSICS and not trent.m_NailMotionEnabled or trent.NoNails or trent:IsNailed() and (#trent.Nails >= 8 or trent:GetPropsInContraption() >= GAMEMODE.MaxPropsInBarricade) then return end

	if NONAILS[tr.MatType or 0] then
		owner:PrintMessage(HUD_PRINTCENTER, NONAILS[tr.MatType])
		return
	end

	if trent:IsValid() then
		for _, nail in pairs(ents.FindByClass("prop_nail")) do
			if nail:GetParent() == trent and nail:GetActualPos():Distance(tr.HitPos) <= 16 then
				owner:PrintMessage(HUD_PRINTCENTER, "Too close to another nail.")
				return
			end
		end
	end

	local aimvec = owner:GetAimVector()

	local trtwo = util.TraceLine({start = tr.HitPos, endpos = tr.HitPos + aimvec * 16, filter = {owner, trent}, mask = MASK_SOLID})

	if trtwo.HitSky then return end

	local ent = trtwo.Entity
	if trtwo.HitWorld or ent:IsValid() and util.IsValidPhysicsObject(ent, trtwo.PhysicsBone) and (ent:GetMoveType() == MOVETYPE_VPHYSICS or ent.m_NailMotionEnabled) and not ent.NoNails then
		if NONAILS[trtwo.MatType or 0] then
			owner:PrintMessage(HUD_PRINTCENTER, NONAILS[trtwo.MatType])
			return
		end

		if ent and ent:IsValid() and (ent.NoNails or ent:IsNailed() and (#ent.Nails >= 8 or ent:GetPropsInContraption() >= GAMEMODE.MaxPropsInBarricade)) then return end

		local cons = constraint.Weld(trent, ent, tr.PhysicsBone, trtwo.PhysicsBone, 0, true)
		if cons ~= nil then
			for _, oldcons in pairs(constraint.FindConstraints(trent, "Weld")) do
				if oldcons.Ent1 == ent or oldcons.Ent2 == ent then
					cons = oldcons.Constraint
					break
				end
			end
		end

		if not cons then return end

		if self.Alternate then
			self:SendWeaponAnim(ACT_VM_HITCENTER)
		else
			self:SendWeaponAnim(ACT_VM_MISSCENTER)
		end
		self.Alternate = not self.Alternate

		owner:DoAnimationEvent(ACT_HL2MP_GESTURE_RANGE_ATTACK_MELEE)

		self:SetNextPrimaryFire(CurTime() + 1)
		self:TakePrimaryAmmo(1)

		trent:EmitSound("weapons/melee/crowbar/crowbar_hit-"..math.random(4)..".wav")

		local nail = ents.Create("prop_nail")
		if nail:IsValid() then
			nail:SetActualOffset(tr.HitPos, trent)
			nail:SetPos(tr.HitPos - aimvec * 8)
			nail:SetAngles(aimvec:Angle())
			nail:AttachTo(trent, ent, tr.PhysicsBone, trtwo.PhysicsBone)
			nail:Spawn()
			nail:SetDeployer(owner)

			if self.PreviousNailHealth[1] then
				nail:SetNailHealth(self.PreviousNailHealth[1])
				table.remove(self.PreviousNailHealth, 1)
			end
			if self.PreviousNailRepairMultiplier[1] then
				nail:SetRepairMultiplier(self.PreviousNailRepairMultiplier[1])
				table.remove(self.PreviousNailRepairMultiplier, 1)
			end

			--nail.constraint = cons
			cons:DeleteOnRemove(nail)

			gamemode.Call("OnNailCreated", trent, ent, nail)
		end
	end
end
