function GM:ForceDermaSkin()
	return "ZS"
end

local surface = surface
local draw = draw
local Color = Color

local SKIN = {}

SKIN.PrintName = "Zombie Survival Derma Skin"
SKIN.Author = "William \"JetBoom\" Moodhe"
SKIN.DermaVersion = 1

SKIN.bg_color = Color(50, 50, 50, 255)
SKIN.bg_color_sleep = Color(40, 40, 40, 255)
SKIN.bg_color_dark = Color(30, 30, 30, 255)
SKIN.bg_color_bright = Color(80, 80, 80, 255)

SKIN.panel_transback = Color(50, 50, 50, 120)
SKIN.tooltip = Color(190, 190, 190, 230)

SKIN.color_frame_background = Color(0, 0, 0, 220) --SKIN.color_frame_background = Color(10, 10, 10, 255)
SKIN.color_frame_border = Color(0, 80, 0, 255)

SKIN.control_color = Color(40, 40, 40, 160)
SKIN.control_color_dark = Color(25, 25, 25, 200)
SKIN.control_color_active = Color(55, 75, 55, 160)
SKIN.control_color_highlight = Color(50, 65, 50, 160)

SKIN.color_textentry_background = Color(40, 40, 40, 255)
SKIN.color_textentry_border = Color(70, 90, 70, 255)

SKIN.color_purewhite = Color(255, 255, 255, 255)

SKIN.colButtonText = Color(240, 255, 240, 255)
SKIN.colButtonTextDisabled = Color(240, 255, 240, 55)
SKIN.colButtonBorder = Color(20, 25, 20, 255)
SKIN.colButtonBorderHighlight = Color(200, 210, 200, 50)
SKIN.colButtonBorderShadow = Color(0, 0, 0, 120)

SKIN.colMenuBG = Color(140, 150, 140, 200)
SKIN.colMenuBorder = Color(0, 0, 0, 200)

SKIN.colPropertySheet = Color(40, 40, 40, 255)
SKIN.colTab = SKIN.colPropertySheet
SKIN.colTabInactive = Color(25, 25, 25, 155)
SKIN.colTabShadow = Color(20, 30, 20, 255)
SKIN.colTabText	= Color(240, 255, 240, 255)
SKIN.colTabTextInactive	= Color(240, 255, 240, 120)

function SKIN:SchemeTreeNodeButton(panel)
	--DLabel.ApplySchemeSettings(panel)
	panel:SetTextColor(self.color_purewhite)
end

local texCorner = surface.GetTextureID("zombiesurvival/circlegradient")
local texUpEdge = surface.GetTextureID("gui/gradient_up")
local texDownEdge = surface.GetTextureID("gui/gradient_down")
local texRightEdge = surface.GetTextureID("gui/gradient")
function SKIN:PaintFrame(panel)
	local wid, hei = panel:GetSize()
	local edgesize = math.ceil(math.min(hei * 0.1, math.min(16, wid * 0.1)))
	local dedgesize = edgesize * 2
	local hedgesize = edgesize * 0.5
	DisableClipping(true)
	surface.SetDrawColor(self.color_frame_background)
	surface.DrawRect(0, 0, wid, hei)
	surface.SetTexture(texUpEdge)
	surface.DrawTexturedRect(0, -edgesize, wid, edgesize)
	surface.SetTexture(texDownEdge)
	surface.DrawTexturedRect(0, hei, wid, edgesize)
	surface.SetTexture(texRightEdge)
	surface.DrawTexturedRect(wid, 0, edgesize, hei)
	surface.DrawTexturedRectRotated(hedgesize * -1, hei * 0.5, edgesize, hei, 180)
	surface.SetTexture(texCorner)
	surface.DrawTexturedRect(-edgesize, -edgesize, edgesize, edgesize)
	surface.DrawTexturedRectRotated(wid + hedgesize, -hedgesize, edgesize, edgesize, 270)
	surface.DrawTexturedRectRotated(wid + hedgesize, hei + hedgesize, edgesize, edgesize, 180)
	surface.DrawTexturedRectRotated(-hedgesize, hei + hedgesize, edgesize, edgesize, 90)
	DisableClipping(false)
end

function SKIN:DrawBorder(x, y, w, h, border)
	surface.SetDrawColor(border)
	surface.DrawOutlinedRect(x, y, w, h)
	surface.SetDrawColor(border.r * 0.75, border.g * 0.75, border.b * 0.5, border.a)
	surface.DrawOutlinedRect(x + 1, y + 1, w - 2, h - 2)
	surface.SetDrawColor(border.r * 0.5, border.g * 0.5, border.b * 0.5, border.a)
	surface.DrawOutlinedRect(x + 2, y + 2, w - 4, h - 4)
end

function SKIN:PaintTooltip(panel)
	local w, h = panel:GetSize()

	DisableClipping(true)

	self:DrawGenericBackground(0, 0, w, h, self.tooltip)
	panel:DrawArrow(0, 0)

	DisableClipping(false)
end

function SKIN:PaintButton(panel)
	local w, h = panel:GetSize()

	if panel.m_bBackground then
		local col = self.control_color
		if panel:GetDisabled() then
			col = self.control_color_dark
		elseif panel.Depressed or panel:GetSelected() then
			col = self.control_color_active
		elseif panel.Hovered then
			col = self.control_color_highlight
		end

		draw.RoundedBox(8, 0, 0, w, h, col)
	end
end

function SKIN:PaintTextEntry(panel)
	if panel.m_bBackground then
		surface.SetDrawColor(self.color_textentry_background)
		surface.DrawRect(0, 0, panel:GetWide(), panel:GetTall())
	end

	panel:DrawTextEntryText(panel.m_colText, panel.m_colHighlight, panel.m_colCursor)

	if panel.m_bBorder then
		self:DrawBorder(0, 0, panel:GetWide(), panel:GetTall(), self.color_textentry_border)
	end	
end

function SKIN:SchemeTextEntry(panel)
	panel:SetTextColor(self.color_purewhite)
	panel:SetHighlightColor(self.color_purewhite)
	panel:SetCursorColor(self.color_purewhite)
end

local oldinit = DTree_Node.Init
function DTree_Node:Init()
	oldinit(self)

	self:SetTextColor(SKIN.color_purewhite)
end

--[[local old = DTooltip.SetContents
function DTooltip:SetContents(...)
	old(self, ...)

	if self.Contents.SetTextColor then
		self.Contents:SetTextColor(SKIN.color_purewhite)
	end
end]]

derma.DefineSkin("ZS", "The default Derma skin for Zombie Survival", SKIN, "Default")
