local meta = FindMetaTable("Entity")
if not meta then return end

function meta:HitFence(data, phys)
	local pos = phys:GetPos()
	local vel = data.OurOldVelocity
	local endpos = data.HitPos + vel:GetNormalized()
	if util.TraceLine({start = pos, endpos = endpos, mask = MASK_SOLID, filter = self}).Hit and not util.TraceLine({start = pos, endpos = endpos, mask = MASK_SHOT, filter = self}).Hit then -- Essentially hit a fence or passable object.
		self:SetPos(data.HitPos)
		phys:SetPos(data.HitPos)
		phys:SetVelocityInstantaneous(vel)

		return true
	end

	return false
end

function meta:GhostAllPlayersInMe()
	if self.GhostedBefore then return end
	self.GhostedBefore = true

	self.OldMaterial = self:GetMaterial()
	self.OldCollisionGroup = self:GetCollisionGroup()
	self:SetCollisionGroup(COLLISION_GROUP_DEBRIS_TRIGGER)
	self:SetMaterial("models/spawn_effect")
	local timnam = "ENABLECOLLISIONS"..tostring(self)
	timer.Create(timnam, 0, 0, EnableCollisions, self, 8, TEAM_HUMAN, timnam)
end

local function SortItems(a, b)
	if a.CleanupPriority == b.CleanupPriority then
		return a.Created < b.Created
	end

	return a.CleanupPriority < b.CleanupPriority
end
local function CheckItemCreated(self)
	if not self:IsValid() or self.PlacedInMap then return end

	local tab = {}
	for _, ent in pairs(ents.FindByClass("prop_ammo")) do
		if not ent.PlacedInMap then
			table.insert(tab, ent)
		end
	end
	for _, ent in pairs(ents.FindByClass("prop_flashlightbattery")) do
		if not ent.PlacedInMap then
			table.insert(tab, ent)
		end
	end
	for _, ent in pairs(ents.FindByClass("prop_weapon")) do
		if not ent.PlacedInMap then
			table.insert(tab, ent)
		end
	end

	if #tab > GAMEMODE.MaxDroppedItems then
		table.sort(tab, SortItems)
		for i = 1, GAMEMODE.MaxDroppedItems do
			tab[i]:Remove()
		end
	end
end
function meta:ItemCreated()
	self.Created = self.Created or CurTime()
	timer.Simple(0, CheckItemCreated, self)
end

function meta:FireOutput(outpt, activator, caller, args)
	local intab = self[outpt]
	if intab then
		for key, tab in pairs(intab) do
			for __, subent in pairs(self:FindByNameHammer(tab.entityname, activator, caller)) do
				if tab.delay == 0 then
					subent:Input(tab.input, activator, caller, tab.args)
				else
					timer.Simple(tab.delay, function() if subent:IsValid() then subent:Input(tab.input, activator, caller, tab.args) end end)
				end
			end
		end
	end
end

function meta:AddOnOutput(key, value)
	self[key] = self[key] or {}
	local tab = string.Explode(",", value)
	table.insert(self[key], {entityname=tab[1], input=tab[2], args=tab[3], delay=tab[4], reps=tab[5]})
end

function meta:FindByNameHammer(name, activator, caller)
	if name == "!self" then return {self} end
	if name == "!activator" then return {activator} end
	if name == "!caller" then return {caller} end
	return ents.FindByName(name)
end

function meta:IsNailed()
	if self.Nails then
		for _, nail in pairs(self.Nails) do
			if nail and nail:IsValid() and (nail:GetAttachEntity() == self or nail:GetBaseEntity() == self) then
				return true
			end
		end
	end

	return false
end

function meta:IsWeldedToWorld()
	local conss = constraint.FindConstraints(self, "Weld")
	if conss then
		for _, cons in pairs(conss) do
			if cons and cons.Entity and ((cons.Entity[1] and cons.Entity[1].World) or (cons.Entity[2] and cons.Entity[2].World)) then return true end
		end
	end

	return false
end

function meta:IsNailedToWorld()
	return self:IsNailed() and self:IsWeldedToWorld()
end

function meta:IsNailedToWorldHierarchy()
	if self:IsNailedToWorld() then
		return true
	end

	local entlist, conlist = duplicator.Copy(self)
	for _, enttab in pairs(entlist) do
		local ent = enttab.Entity
		if ent and ent:IsValid() then
			if ent:IsNailedToWorld() then return true end
		end
	end

	return false
end

function meta:PackUp(pl)
	if not self.CanPackUp then return end

	local cur = pl:GetStatus("packup")
	if cur and cur:IsValid() then return end

	local status = pl:GiveStatus("packup")
	if status:IsValid() then
		status:SetPackUpEntity(self)
		status:SetEndTime(CurTime() + (self.PackUpTime or 4))
	end
end

function meta:GetPropsInContraption()
	local count = 0

	local entlist, conlist = duplicator.Copy(self)
	for _, enttab in pairs(entlist) do
		local ent = enttab.Entity
		if ent and ent:IsValid() then
			count = count + 1
		end
	end

	return count
end

function meta:ResetLastBarricadeAttacker(attacker)
	if attacker:IsPlayer() and attacker:Team() == TEAM_UNDEAD then
		self.m_LastDamagedByZombie = CurTime()
	end
end

meta.OldSetPhysicsAttacker = meta.SetPhysicsAttacker
function meta:SetPhysicsAttacker(ent)
	if self:GetClass() == "func_physbox" and ent:IsValid() then
		self.PBAttacker = ent
		self.NPBAttacker = CurTime() + 1
	end
	self:OldSetPhysicsAttacker(ent)
end

local function randomsort(a, b)
	return a.rand < b.rand
end
local function randomize(t)
	for k, v in pairs(t) do v.rand = math.Rand(0, 1) end
	table.sort(t, randomsort)
end

-- Return true to override default behavior.
function meta:DamageNails(attacker, inflictor, damage, dmginfo)
	if not self:IsNailed() or self.m_NailsDontAbsorb then return end

	if not gamemode.Call("CanDamageNail", self, attacker, inflictor, damage, dmginfo) then
		if dmginfo then
			dmginfo:SetDamage(0)
			dmginfo:SetDamageType(DMG_BULLET)
		end

		return true
	end

	self:ResetLastBarricadeAttacker(attacker)

	local nails = self:GetNails()
	randomize(nails)
	local numnails = #nails
	if numnails > 0 then
		local damagescaleleft = 1

		for i, nail in ipairs(nails) do
			local scale = 1 / numnails
			local damagescale = i == numnails and damagescaleleft or math.Rand(scale / numnails, scale)
			damagescaleleft = damagescaleleft - damagescale
			local thisdamage = damage * damagescale
			if dmginfo then dmginfo:SetDamage(thisdamage) end

			nail:SetNailHealth(nail:GetNailHealth() - thisdamage)

			nail:OnDamaged(thisdamage, attacker, inflictor, dmginfo)

			gamemode.Call("OnNailDamaged", self, attacker, inflictor, thisdamage, dmginfo)
			if nail:GetNailHealth() <= 0 then
				gamemode.Call("OnNailDestroyed", self, attacker, inflictor, thisdamage, dmginfo)
				self:RemoveNail(nail)
			end
		end

		if dmginfo then dmginfo:SetDamage(0) end
		return true
	end
end

function meta:GetNails()
	local tab = {}

	if self.Nails then
		for _, nail in pairs(self.Nails) do
			if nail and nail:IsValid() then
				tab[#tab + 1] = nail
			end
		end
	end

	return tab
end

function meta:GetFirstNail()
	if self.Nails then
		for i, nail in ipairs(self.Nails) do
			if nail and nail:IsValid() and not nail:GetAttachEntity():IsValid() then return nail end
		end
		for i, nail in ipairs(self.Nails) do
			if nail and nail:IsValid() then return nail end
		end
	end
end

local function GetNailOwner(nail, filter)
	for _, ent in pairs(ents.GetAll()) do
		if ent ~= filter and ent.Nails then
			for __, n in pairs(ent.Nails) do
				if n == nail then
					return ent
				end
			end
		end
	end

	return GetWorldEntity()
end

function meta:RemoveNail(nail, dontremoveentity, removedby)
	if not self:IsNailed() then return end

	if not nail then
		nail = self:GetFirstNail()
	end

	if not nail or not nail:IsValid() then return end

	local cons = nail:GetNailConstraint()
	local othernails = 0
	for _, othernail in pairs(ents.FindByClass("prop_nail")) do
		if othernail ~= nail and othernail:GetNailConstraint():IsValid() and othernail:GetNailConstraint() == cons then
			othernails = othernails + 1
		end
	end

	-- Only remove the constraint if it's the last nail.
	if othernails == 0 then
		cons:Remove()
	end

	local ent2 = GetNailOwner(nail, self)

	for i, n in ipairs(self.Nails) do
		if n == nail then
			table.remove(self.Nails, i)
			break
		end
	end

	if ent2 and ent2.Nails then
		for i, n in ipairs(ent2.Nails) do
			if n == nail then
				table.remove(ent2.Nails, i)
				ent2:TemporaryBarricadeObject()
				break
			end
		end
	end

	--self:CollisionRulesChanged()
	self:TemporaryBarricadeObject()

	gamemode.Call("OnNailRemoved", nail, self, ent2, removedby)

	if not dontremoveentity then
		nail.m_IsRemoving = true
		nail:Remove()
	end

	return true
end
