GM.ZombieClasses = {}
GM.DefaultZombieClass = GM.DefaultZombieClass or 1

function GM:IsClassUnlocked(classname)
	local classtab = self.ZombieClasses[classname]
	if not classtab then return false end

	if classtab.IsClassUnlocked then
		local ret = classtab:IsClassUnlocked()
		if ret ~= nil then return ret end
	end

	return not classtab.Locked and (classtab.Unlocked or classtab.Wave and self:GetWave() >= classtab.Wave or not self:GetWaveActive() and self:GetWave() + 1 >= classtab.Wave)
end

local function ReorderZombieClassesSort(a, b)
	if (a.Order or b.Order) and a.Order ~= b.Order then
		return (a.Order or 255) < (b.Order or 255)
	end

	if (a.Wave or b.Wave) and a.Wave ~= b.Wave then
		return (a.Wave or 255) < (b.Wave or 255)
	end

	return a.Name < b.Name
end
function GM:ReorderZombieClasses()
	table.sort(self.ZombieClasses, ReorderZombieClassesSort)
	for k, v in pairs(self.ZombieClasses) do
		if type(k) == "number" then
			self.ZombieClasses[v.Name] = v
			v.Index = k

			if v.IsDefault then
				self.DefaultZombieClass = k
			end
		end
	end
end

function GM:RegisterZombieClass(name, tab)
	local gm = GAMEMODE or GM

	tab.Wave = math.floor(tab.Wave * self:GetNumberOfWaves())
	table.insert(gm.ZombieClasses, tab)
	tab.Index = #gm.ZombieClasses

	if tab.IsDefault then
		gm.DefaultZombieClass = tab.Index
	end

	gm.ZombieClasses[name] = gm.ZombieClasses[tab.Index]
end

function GM:RegisterZombieClasses()
	local classes = file.FindInLua(self.FolderName.."/gamemode/zombieclasses/*.lua")
	table.sort(classes)
	for i, filename in ipairs(classes) do
		AddCSLuaFile("zombieclasses/"..filename)
		CLASS = {}
		include("zombieclasses/"..filename)
		if CLASS.Name then
			self:RegisterZombieClass(CLASS.Name, CLASS)
		else
			ErrorNoHalt("CLASS "..filename.." has no 'Name' member!")
		end
		CLASS = nil
	end

	self:ReorderZombieClasses()
end

GM:RegisterZombieClasses()
