GM.Name		=	"Zombie Survival"
GM.Version	=	363
GM.Author	=	"William \"JetBoom\" Moodhe"
GM.Email	=	"jetboom@yahoo.com"
GM.Website	=	"http://www.noxiousnet.com"

GM.Credits = {
	{"William \"JetBoom\" Moodhe", "jetboom@yahoo.com (www.noxiousnet.com)", "Creator / Programmer"},
	{"11k", "tjd113@gmail.com", "Zombie view models"},
	{"Eisiger", "k2deseve@gmail.com", "Zombie kill icons"},
	{"Austin \"Little Nemo\" Killey", "austin_odyssey@yahoo.com", "Default beat set sounds"},
	{"Zombie Panic: Source", "http://www.zombiepanic.org/", "Melee weapon models"},
	{"Samuel", "samuel_games@hotmail.com", "Board Kit model"},
	{"Typhon", "lukas-tinel@hotmail.com", "HUD textures"}
}

include("sh_colors.lua")
include("sh_serialization.lua")

include("sh_globals.lua")
include("sh_util.lua")
include("sh_options.lua")
include("sh_zombieclasses.lua")
include("sh_animations.lua")

include("obj_entity_extend.lua")
include("obj_player_extend.lua")
include("obj_weapon_extend.lua")

GM.EndRound = false

team.SetUp(TEAM_ZOMBIE, "The Undead", Color(0, 255, 0, 255))
team.SetUp(TEAM_SURVIVORS, "Survivors", Color(0, 160, 255, 255))

function GM:SetRedeemBrains(amount)
	SetGlobalInt("redeembrains", amount)
end

function GM:GetRedeemBrains()
	return GetGlobalInt("redeembrains", self.DefaultRedeem)
end

function GM:PlayerIsAdmin(pl)
	return pl:IsAdmin()
end

function GM:GetFallDamage(pl, fallspeed)
	return 0
end

function GM:ZombieSpawnDistanceSort(other)
	return self._ZombieSpawnDistance < other._ZombieSpawnDistance
end

function GM:SortZombieSpawnDistances(allplayers)
	local zspawns = ents.FindByClass("zombiegasses") --team.GetSpawnPoint(TEAM_UNDEAD)
	for _, pl in pairs(allplayers) do
		if pl:Team() == TEAM_UNDEAD or pl:GetNetworkedBool("pvolunteer", false) then
			pl._ZombieSpawnDistance = -1
		else
			local plpos = pl:GetPos()
			local closest = 9999999
			for _, ent in pairs(zspawns) do
				local dist = ent:GetPos():Distance(plpos)
				if dist < closest then
					closest = dist
				end
			end
			pl._ZombieSpawnDistance = closest
		end
	end

	table.sort(allplayers, self.ZombieSpawnDistanceSort)
end

function GM:SetDynamicSpawning(onoff)
	SetGlobalBool("DynamicSpawningDisabled", not onoff)
	self.DynamicSpawning = onoff
end

local playerheight = Vector(0, 0, 72)
local playermins = Vector(-17, -17, 0)
local playermaxs = Vector(17, 17, 4)

function GM:DynamicSpawnIsValid(zombie, humans, allplayers)
	-- Optional caching for these.
	if not humans then humans = team.GetPlayers(TEAM_HUMAN) end
	if not allplayers then allplayers = player.GetAll() end

	local pos = zombie:GetPos() + Vector(0, 0, 1)
	if zombie:Alive() and zombie:GetMoveType() == MOVETYPE_WALK and not util.TraceHull({start = pos, endpos = pos + playerheight, mins = playermins, maxs = playermaxs, mask = MASK_SOLID, filter = allplayers}).Hit then
		local valid = true

		for ___, human in pairs(humans) do
			local eyepos = human:EyePos()
			local nearest = zombie:NearestPoint(eyepos)
			local dist = eyepos:Distance(nearest)
			if dist <= 768 or dist <= 2048 and WorldVisible(eyepos, nearest) then -- Zombies can't be in radius of any humans. Zombies can't be clearly visible by any humans.
				valid = false
				break
			end
		end

		return valid
	end

	return false
end

function GM:GetBestDynamicSpawn(pl)
	local spawns = self:GetDynamicSpawns(pl)
	if #spawns == 0 then return end

	return self:GetClosestSpawnPoint(spawns, self:GetTeamEpicentre(TEAM_HUMAN)) or table.Random(spawns)
end

function GM:GetDynamicSpawns(pl)
	local tab = {}

	local allplayers = player.GetAll()
	local humans = team.GetPlayers(TEAM_HUMAN)
	for _, zombie in pairs(team.GetPlayers(TEAM_UNDEAD)) do
		local pos = zombie:GetPos() + Vector(0, 0, 1)
		if zombie ~= pl and self:DynamicSpawnIsValid(zombie, humans, allplayers) then
			table.insert(tab, zombie)
		end
	end

	return tab
end

function GM:GetDesiredStartingZombies()
	local numplayers = #player.GetAll()
	return math.min(math.max(1, math.ceil(numplayers * self.WaveOneZombies)), numplayers - 1)
end

function GM:GetEndRound()
	return self.EndRound
end

function GM:PrecacheResources()
	util.PrecacheSound("physics/body/body_medium_break2.wav")
	util.PrecacheSound("physics/body/body_medium_break3.wav")
	util.PrecacheSound("physics/body/body_medium_break4.wav")
	for name, mdl in pairs(player_manager.AllValidModels()) do
		util.PrecacheModel(mdl)
	end
end

function GM:ShouldCollide(enta, entb)
	if enta.ShouldNotCollide and enta:ShouldNotCollide(entb) or entb.ShouldNotCollide and entb:ShouldNotCollide(enta) then
		return false
	end

	return true
end

local function SkewedDistance(a, b, zskew)
	return math.sqrt((b.x - a.x) ^ 2 + (b.y - a.y) ^ 2 + (zskew * (b.z - a.z)) ^ 2)
end

function GM:Move(pl, move)
	if pl.Move and pl.Move(pl.ClassTable, pl, move) then
		return
	end

	if pl:Team() == TEAM_HUMAN then
		if pl:GetBarricadeGhosting() then
			move:SetMaxSpeed(36)
			move:SetMaxClientSpeed(36)
		else
			local speed = move:GetForwardSpeed()
			local sidespeed = move:GetSideSpeed()
			if speed < -88 then
				move:SetForwardSpeed(math.min(-88, speed * 0.45))
			
				if sidespeed < -88 then
					move:SetSideSpeed(math.min(-88, sidespeed * 0.45))
				elseif 88 < sidespeed then
					move:SetSideSpeed(math.max(88, sidespeed * 0.45))
				end
			end
		end
	end

	local legdamage = pl:GetLegDamage()
	if legdamage > 0 then
		local scale = 1 - math.min(1, legdamage * 0.33)
		move:SetSideSpeed(move:GetSideSpeed() * scale)
		move:SetForwardSpeed(move:GetForwardSpeed() * scale)
	end
end

function GM:PlayerCanBeHealed(pl)
	return not pl:GetHemophilia()
end

function GM:PlayerCanPurchase(pl)
	return pl:IsValid() and pl:Team() == TEAM_HUMAN and pl:Alive() and self:GetWave() > 0 and pl:NearArsenalCrate()
end

function GM:PlayerTraceAttack(pl, dmginfo, dir, trace)
end

function GM:ScalePlayerDamage(pl, hitgroup, dmginfo, virtual, noreduce)
	if virtual then
		if not noreduce then
			dmginfo:SetDamage(dmginfo:GetDamage() * 0.5)
		end

		if not pl:CallZombieFunction("ScalePlayerDamage", pl, hitgroup, dmginfo) then
			if hitgroup == HITGROUP_HEAD then
				dmginfo:SetDamage(dmginfo:GetDamage() * 2)
			elseif hitgroup == HITGROUP_LEFTARM or hitgroup == HITGROUP_RIGHTARM or hitgroup == HITGROUP_LEFTLEG or hitgroup == HITGROUP_RIGHTLEG or hitgroup == HITGROUP_GEAR then
				dmginfo:SetDamage(dmginfo:GetDamage() * 0.25)
			elseif hitgroup == HITGROUP_STOMACH then
				dmginfo:SetDamage(dmginfo:GetDamage() * 0.75)
			end
		end

		if (hitgroup == HITGROUP_LEFTLEG or hitgroup == HITGROUP_RIGHTLEG) and self:PlayerShouldTakeDamage(pl, dmginfo:GetAttacker()) then
			pl:AddLegDamage(dmginfo:GetDamage())
		end
	end
end

function GM:ScaleNPCDamage(npc, hitgroup, dmginfo)
    if hitgroup == HITGROUP_HEAD then
		dmginfo:SetDamage(dmginfo:GetDamage() * 2)
	elseif hitgroup == HITGROUP_LEFTARM or hitgroup == HITGROUP_RIGHTARM or hitgroup == HITGROUP_LEFTLEG or hitgroup == HITGROUP_RIGHTLEG or hitgroup == HITGROUP_GEAR then
		dmginfo:SetDamage(dmginfo:GetDamage() * 0.5)
	elseif hitgroup ~= HITGROUP_CHEST then
		dmginfo:SetDamage(dmginfo:GetDamage() * 0.75)
	end
end

function GM:CanDamageNail(ent, attacker, inflictor, damage, dmginfo)
	return not attacker:IsPlayer() or attacker:Team() ~= TEAM_HUMAN
end

function GM:GetZombieDamageScale(pos, ignore)
	return self.ZombieDamageMultiplier --(1 - self:GetFearMeterPower(pos, TEAM_UNDEAD, ignore) * 0.3) * self.ZombieDamageMultiplier
end

local temppos
local function SortByDistance(a, b)
	return a:GetPos():Distance(temppos) < b:GetPos():Distance(temppos)
end

function GM:GetClosestSpawnPoint(teamid, pos)
	temppos = pos
	local spawnpoints
	if type(teamid) == "table" then
		spawnpoints = teamid
	else
		spawnpoints = team.GetSpawnPoint(teamid)
	end
	table.sort(spawnpoints, SortByDistance)
	return spawnpoints[1]
end

local FEAR_RANGE = 768
local FEAR_PERINSTANCE = 0.1
local RALLYPOINT_THRESHOLD = 0.3

local function GetEpicenter(tab)
	local vec = Vector(0, 0, 0)
	if #tab == 0 then return vec end

	for k, v in pairs(tab) do
		vec = vec + v:GetPos()
	end

	return vec / #tab
end

function GM:GetTeamRallyGroups(teamid)
	local groups = {}
	local ingroup = {}

	local plys = team.GetPlayers(teamid)

	for _, pl in pairs(plys) do
		if not ingroup[pl] and pl:Alive() then
			local plpos = pl:GetPos()
			local group = {pl}

			for __, otherpl in pairs(plys) do
				if otherpl ~= pl and not ingroup[otherpl] and otherpl:Alive() and otherpl:GetPos():Distance(plpos) <= FEAR_RANGE then
					group[#group + 1] = otherpl
				end
			end

			if #group * FEAR_PERINSTANCE >= RALLYPOINT_THRESHOLD then
				for k, v in pairs(group) do
					ingroup[v] = true
				end
				groups[#groups + 1] = group
			end
		end
	end

	return groups
end

function GM:GetTeamRallyPoints(teamid)
	local points = {}

	for _, group in pairs(self:GetTeamRallyGroups(teamid)) do
		points[#points + 1] = {GetEpicenter(group), math.min(1, (#group * FEAR_PERINSTANCE - RALLYPOINT_THRESHOLD) / (1 - RALLYPOINT_THRESHOLD))}
	end

	return points
end

local CachedTeamRallyPointTimes = {}
local CachedTeamRallyPoints = {}
function GM:GetCachedTeamRallyPoints(teamid)
	if CachedTeamRallyPoints[teamid] and CurTime() < CachedTeamRallyPointTimes[teamid] then
		return CachedTeamRallyPoints[teamid]
	end

	local points = self:GetTeamRallyPoints(teamid)

	CachedTeamRallyPoints[teamid] = points
	CachedTeamRallyPointTimes[teamid] = CurTime() + 0.1

	return points
end

local CachedEpicentreTimes = {}
local CachedEpicentres = {}
function GM:GetTeamEpicentre(teamid, nocache)
	if not nocache and CachedEpicentres[teamid] and CurTime() < CachedEpicentreTimes[teamid] then
		return CachedEpicentres[teamid]
	end

	local plys = team.GetPlayers(teamid)
	local vVec = Vector(0, 0, 0)
	for _, pl in pairs(plys) do
		if pl:Alive() then
			vVec = vVec + pl:GetPos()
		end
	end

	local epicentre = vVec / #plys
	if not nocache then
		CachedEpicentreTimes[teamid] = CurTime() + 0.5
		CachedEpicentres[teamid] = epicentre
	end

	return epicentre
end
GM.GetTeamEpicenter = GM.GetTeamEpicentre

function GM:GetFearMeterPower(pos, teamid, ignore)
	local power = 0

	for _, pl in pairs(player.GetAll()) do
		if pl ~= ignore and pl:Team() == teamid and not pl:CallZombieFunction("DoesntGiveFear") and pl:Alive() then
			local dist = pl:NearestPoint(pos):Distance(pos)
			if dist <= FEAR_RANGE then
				power = power + ((FEAR_RANGE - dist) / FEAR_RANGE) * FEAR_PERINSTANCE
			end
		end
	end

	return math.min(1, power)
end

function GM:GetRagdollEyes(pl)
	local Ragdoll = pl:GetRagdollEntity()
	if not Ragdoll then return end

	local att = Ragdoll:GetAttachment(Ragdoll:LookupAttachment("eyes"))
	if att then
		att.Pos = att.Pos + att.Ang:Forward() * -2
		att.Ang = att.Ang

		return att.Pos, att.Ang
	end
end

function GM:PlayerNoClip(pl, on)
	if pl:IsAdmin() then
		if SERVER then
			PrintMessage(HUD_PRINTCONSOLE, pl:Name().." turned "..(on and "on" or "off").." noclip.")
		end
		return true
	end

	return false
end

function GM:GetWaveEnd()
	return GetGlobalFloat("waveend", 0)
end

function GM:SetWaveEnd(wave)
	SetGlobalFloat("waveend", wave)
	if SERVER and wav == 1 then
		self:SetRandomsToZombie()
	end
end

function GM:GetWaveStart()
	return GetGlobalFloat("wavestart", self.WaveZeroLength)
end

function GM:SetWaveStart(wave)
	SetGlobalFloat("wavestart", wave)
end

function GM:GetWave()
	return GetGlobalInt("wave", 0)
end

if GM:GetWave() == 0 then
	GM:SetWaveStart(GM.WaveZeroLength)
	GM:SetWaveEnd(GM.WaveZeroLength + GM.WaveOneLength)
end

function GM:GetWaveActive()
	return GetGlobalBool("waveactive", false)
end

function GM:SetWaveActive(active)
	if self.RoundEnded then return end

	if self:GetWaveActive() ~= active then
		SetGlobalBool("waveactive", active)

		if SERVER then
			gamemode.Call("WaveStateChanged", active)
		end
	end
end

function GM:PlayerFootstep(pl, vPos, iFoot, strSoundName, fVolume, pFilter)
end

function GM:PlayerStepSoundTime(pl, iType, bWalking)
	local fStepTime = 350

	if iType == STEPSOUNDTIME_NORMAL or iType == STEPSOUNDTIME_WATER_FOOT then
		local fMaxSpeed = pl:GetMaxSpeed()
		if fMaxSpeed <= 100 then
			fStepTime = 400
		elseif fMaxSpeed <= 300 then
			fStepTime = 350
		else
			fStepTime = 250
		end
	elseif iType == STEPSOUNDTIME_ON_LADDER then
		fStepTime = 450
	elseif iType == STEPSOUNDTIME_WATER_KNEE then
		fStepTime = 600
	end

	if pl:Crouching() then
		fStepTime = fStepTime + 50
	end

	return fStepTime
end