local PANEL = {}

function PANEL:Init()
	self.m_HumanCount = vgui.Create("DTeamCounter", self)
	self.m_HumanCount:SetTeam(TEAM_HUMAN)
	self.m_HumanCount:SetImage("zombiesurvival/humanhead")

	self.m_ZombieCount = vgui.Create("DTeamCounter", self)
	self.m_ZombieCount:SetTeam(TEAM_UNDEAD)
	self.m_ZombieCount:SetImage("zombiesurvival/zombiehead")

	self.m_Text1 = vgui.Create("DLabel", self)
	self.m_Text2 = vgui.Create("DLabel", self)
	self.m_Text3 = vgui.Create("DLabel", self)
	self:SetTextFont("ZSHUDFontTiny")

	self.m_Text1.Paint = self.Text1Paint
	self.m_Text2.Paint = self.Text2Paint
	self.m_Text3.Paint = self.Text3Paint

	self:InvalidateLayout()
end

function PANEL:SetTextFont(font)
	self.m_Text1.Font = font
	self.m_Text1:SetFont(font)
	self.m_Text2.Font = font
	self.m_Text2:SetFont(font)
	self.m_Text3.Font = font
	self.m_Text3:SetFont(font)

	self:InvalidateLayout()
end

function PANEL:PerformLayout()
	local hs = self:GetTall() * 0.5
	self.m_HumanCount:SetSize(hs, hs)
	self.m_ZombieCount:SetSize(hs, hs)
	self.m_ZombieCount:AlignTop(hs)

	self.m_Text1:SetWide(self:GetWide())
	self.m_Text1:SizeToContentsY()
	self.m_Text1:MoveRightOf(self.m_HumanCount, 12)
	self.m_Text1:AlignTop(4)
	self.m_Text2:SetWide(self:GetWide())
	self.m_Text2:SizeToContentsY()
	self.m_Text2:MoveRightOf(self.m_HumanCount, 12)
	self.m_Text2:CenterVertical()
	self.m_Text3:SetWide(self:GetWide())
	self.m_Text3:SizeToContentsY()
	self.m_Text3:MoveRightOf(self.m_HumanCount, 12)
	self.m_Text3:AlignBottom(4)
end

function PANEL:Text1Paint()
	local wave = GAMEMODE:GetWave()
	if wave <= 0 then
		draw.SimpleText("Prepare yourselves...", self.Font, 0, 0, COLOR_GRAY)
	else
		local maxwaves = GAMEMODE:GetNumberOfWaves()
		if maxwaves ~= -1 then
			if GAMEMODE:GetWaveActive() then
				draw.SimpleText("Wave ".. wave .. " of ".. maxwaves, self.Font, 0, 0, COLOR_GRAY)
			else
				draw.SimpleText("Intermission - Wave ".. wave .. " of ".. maxwaves, self.Font, 0, 0, COLOR_GRAY)
			end
		elseif not GAMEMODE:GetWaveActive() then
			draw.SimpleText("Intermission", self.Font, 0, 0, COLOR_GRAY)
		end
	end

	return true
end

function PANEL:Text2Paint()
	if GAMEMODE:GetWave() <= 0 then
		local timeleft = math.max(0, GAMEMODE:GetWaveStart() - CurTime())
		if timeleft < 10 then
			local glow = math.sin(RealTime() * 8) * 200 + 255
			draw.SimpleText("Zombie invasion: "..string.ToMinutesSeconds(timeleft), self.Font, 0, 0, Color(255, glow, glow))
		else
			draw.SimpleText("Zombie invasion: "..string.ToMinutesSeconds(timeleft), self.Font, 0, 0, COLOR_GRAY)
		end
	elseif GAMEMODE:GetWaveActive() then
		local waveend = GAMEMODE:GetWaveEnd()
		if waveend ~= -1 then
			local timeleft = math.max(0, waveend - CurTime())
			draw.SimpleText("Wave ends: "..string.ToMinutesSeconds(timeleft), self.Font, 0, 0, 10 < timeleft and COLOR_GRAY or Color(255, 0, 0, math.abs(math.sin(RealTime() * 8)) * 180 + 40))
		end
	else
		local wavestart = GAMEMODE:GetWaveStart()
		if wavestart ~= -1 then
			local timeleft = math.max(0, wavestart - CurTime())
			draw.SimpleText("Next wave: "..string.ToMinutesSeconds(timeleft), self.Font, 0, 0, 10 < timeleft and COLOR_GRAY or Color(255, 0, 0, math.abs(math.sin(RealTime() * 8)) * 180 + 40))
		end
	end

	return true
end

function PANEL:Text3Paint()
	if MYSELFVALID then
		if MySelf:Team() == TEAM_UNDEAD then
			local toredeem = GAMEMODE:GetRedeemBrains()
			if toredeem > 0 then
				draw.SimpleText("Brains eaten: "..MySelf:Frags().." / "..toredeem, self.Font, 0, 0, COLOR_DARKRED)
			else
				draw.SimpleText("Brains eaten: "..MySelf:Frags(), self.Font, 0, 0, COLOR_DARKRED)
			end
		else
			draw.SimpleText("Points: "..MySelf:GetPoints().." / "..MySelf:Frags(), self.Font, 0, 0, COLOR_DARKRED)
		end
	end

	return true
end

--local texGradRight = surface.GetTextureID("VGUI/gradient-r")
function PANEL:Paint()
	--[[surface.SetTexture(texGradRight)
	surface.SetDrawColor(0, 0, 0, 255)
	surface.DrawTexturedRectRotated(0, self:GetTall() / 2, self:GetWide() * 2, self:GetTall(), 180)]]

	return true
end

vgui.Register("DGameState", PANEL, "DPanel")
