local PANEL = {}
PANEL.m_NextRefresh = 0
PANEL.HumanScroll = 0
PANEL.UndeadScroll = 0

function PANEL:Init()
	self.m_NextRefresh = RealTime() + 0.01

	self.m_F1Help = EasyLabel(self, "F1: Help", "DefaultSmall")
	self.m_F2Help = EasyLabel(self, "F2: Worth menu / Redeem", "DefaultSmall")
	self.m_F3Help = EasyLabel(self, "F3: Upgrades tree / Change Zombie class", "DefaultSmall")
	self.m_F4Help = EasyLabel(self, "F4: Options", "DefaultSmall")

	self.m_AuthorLabel1 = EasyLabel(self, "Gamemode created by "..GAMEMODE.Author, "DefaultSmall")
	self.m_AuthorLabel2 = EasyLabel(self, GAMEMODE.Email, "DefaultSmall")
	self.m_AuthorLabel3 = EasyLabel(self, GAMEMODE.Website, "DefaultSmall")

	self.m_HeadingLabel = EasyLabel(self, GAMEMODE.Name, "ZSHUDFont", COLOR_LIMEGREEN)
	self.m_VersionLabel = EasyLabel(self, "v"..GAMEMODE.Version, "ZSHUDFontSmall")
	self.m_ServerNameLabel = EasyLabel(self, GetGlobalString("servername"), "ZSHUDFontSmall")

	local list = vgui.Create("DPanelList", self)
	list:EnableVerticalScrollbar()
	list:EnableHorizontal(false)
	list:SetSpacing(2)
	list:SetPadding(2)
	self.m_HumanPanelList = list

	local list = vgui.Create("DPanelList", self)
	list:EnableVerticalScrollbar()
	list:EnableHorizontal(false)
	list:SetSpacing(2)
	list:SetPadding(2)
	self.m_UndeadPanelList = list

	self:InvalidateLayout()
end

function PANEL:PerformLayout()
	self.BaseClass.PerformLayout(self)

	self.m_F1Help:AlignLeft(8)
	self.m_F1Help:AlignTop(8)
	self.m_F2Help:AlignLeft(8)
	self.m_F2Help:MoveBelow(self.m_F1Help, 2)
	self.m_F3Help:AlignLeft(8)
	self.m_F3Help:MoveBelow(self.m_F2Help, 2)
	self.m_F4Help:AlignLeft(8)
	self.m_F4Help:MoveBelow(self.m_F3Help, 2)

	self.m_AuthorLabel1:AlignRight(8)
	self.m_AuthorLabel1:AlignTop(8)
	self.m_AuthorLabel2:AlignRight(8)
	self.m_AuthorLabel2:MoveBelow(self.m_AuthorLabel1, 2)
	self.m_AuthorLabel3:AlignRight(8)
	self.m_AuthorLabel3:MoveBelow(self.m_AuthorLabel2, 2)

	local wid, hei = self:GetSize()
	local halfwid = wid * 0.5
	local listwid = halfwid - 16
	local y = 8

	self.m_HeadingLabel:AlignTop(8)
	self.m_HeadingLabel:CenterHorizontal()
	self.m_VersionLabel:MoveBelow(self.m_HeadingLabel, 8)
	self.m_VersionLabel:CenterHorizontal()
	self.m_ServerNameLabel:MoveBelow(self.m_VersionLabel, 8)
	self.m_ServerNameLabel:CenterHorizontal()

	self.m_HumanPanelList:SetWide(wid * 0.5 - 16)
	self.m_HumanPanelList:AlignLeft(8)
	self.m_HumanPanelList:MoveBelow(self.m_ServerNameLabel, 8)
	local x, y = self.m_HumanPanelList:GetPos()
	self.m_HumanPanelList:SetTall(hei - y - 8)

	self.m_UndeadPanelList:SetSize(self.m_HumanPanelList:GetSize())
	self.m_UndeadPanelList:AlignRight(8)
	self.m_UndeadPanelList:MoveBelow(self.m_ServerNameLabel, 8)

	timer.Simple(0, self.m_HumanPanelList.VBar.SetScroll, self.m_HumanPanelList.VBar, self.HumanScroll)
	timer.Simple(0, self.m_UndeadPanelList.VBar.SetScroll, self.m_UndeadPanelList.VBar, self.UndeadScroll)
end

function PANEL:Think()
	if self.m_NextRefresh <= RealTime() then
		self.m_NextRefresh = RealTime() + 2
		self:Refresh()
	end
end

function PANEL:Paint()
	--[[local wid, hei = self:GetSize()
	surface.SetDrawColor(10, 10, 10, 255)
	surface.DrawRect(0, 0, wid, hei)
	surface.SetDrawColor(0, 80, 0, 255)
	surface.DrawOutlinedRect(1, 1, wid - 2, hei - 2)
	surface.SetDrawColor(0, 60, 0, 255)
	surface.DrawOutlinedRect(2, 2, wid - 4, hei - 4)
	surface.SetDrawColor(0, 40, 0, 255)
	surface.DrawOutlinedRect(3, 3, wid - 6, hei - 6)]]
	derma.SkinHook("Paint", "Frame", self)
end

function PANEL:RefreshHeading()
	self.m_ServerNameLabel:SetText(GetGlobalString("servername"))
	self.m_ServerNameLabel:SizeToContents()

	self:InvalidateLayout()
end

function PANEL:RefreshHumanList()
	self.HumanScroll = self.m_HumanPanelList.VBar:GetScroll()
	self:RefreshList(self.m_HumanPanelList, TEAM_HUMAN)
end

function PANEL:RefreshUndeadList()
	self.UndeadScroll = self.m_UndeadPanelList.VBar:GetScroll()
	self:RefreshList(self.m_UndeadPanelList, TEAM_UNDEAD)
end

local function PlayerSort(a, b)
	local afrags, bfrags = a:Frags(), b:Frags()
	if afrags == bfrags then
		local adeaths, bdeaths = a:Deaths(), b:Deaths()
		if adeaths == bdeaths then
			return a:UserID() < b:UserID()
		end
		return adeaths < bdeaths
	end
	return afrags > bfrags
end
function PANEL:RefreshList(list, teamid)
	list:Clear(true)

	local pls = team.GetPlayers(teamid)

	local tall = 40
	if 14 < #pls then
		tall = 24
	elseif 8 < #pls then
		tall = 32
	end
	local halftall = tall * 0.5

	local fnt = "Scoreboard"..tall

	if not _G[fnt] then
		surface.CreateFont("typenoksidi", tall - 4, 300, true, true, fnt)
		_G[fnt] = tall
	end

	table.sort(pls, PlayerSort)
	for i, pl in ipairs(pls) do
		local panel = vgui.Create(pl:Team() == TEAM_UNDEAD and "DZSPlayerZombiePanel" or "DZSPlayerPanel")
		panel:SetTall(tall)
		list:AddItem(panel)
		panel:SetPlayer(pl)
	end
end

function PANEL:Refresh()
	self:RefreshHeading()
	self:RefreshHumanList()
	self:RefreshUndeadList()
end
vgui.Register("DZSScoreBoard", PANEL, "DPanel")
