CLASS.Name = "Fast Zombie"
CLASS.Description = "This boney cadaver is much faster than other zombies. They aren't much of a threat by themselves but can reach nearly any area by climbing with their razor sharp claws and they have no problem hunting down weak or hurt humans."

CLASS.Model = Model("models/Zombie/Fast.mdl")

CLASS.Wave = 1 / 2
CLASS.Infliction = 1 / 2

CLASS.Health = 125
CLASS.Speed = 280
CLASS.SWEP = "weapon_zs_fastzombie"

CLASS.KnockbackScale = 0.35

CLASS.Frags = 4

CLASS.Hull = {Vector(-16, -16, 0), Vector(16, 16, 58)}
CLASS.HullDuck = {Vector(-16, -16, 0), Vector(16, 16, 32)}
CLASS.ViewOffset = Vector(0, 0, 50)
CLASS.ViewOffsetDucked = Vector(0, 0, 24)

CLASS.PainSounds = {"NPC_FastZombie.Pain"}
CLASS.DeathSounds = {"NPC_FastZombie.Die"}

CLASS.VoicePitch = 0.75

CLASS.NoFallDamage = true
CLASS.NoFallSlowdown = true

function CLASS:Move(pl, mv)
	local wep = pl:GetActiveWeapon()
	if wep.Move and wep:Move(mv) then
		return true
	end
end

function CLASS:CalcMainActivity(pl, velocity)
	local wep = pl:GetActiveWeapon()
	if not wep:IsValid() or not wep.GetClimbing then return end

	if wep:GetClimbing() then
		pl.CalcSeqOverride = 10
		return true
	elseif wep:GetSwinging() then
		pl.CalcSeqOverride = 8
		return true
	elseif wep:GetPounceTime() > 0 then
		pl.CalcSeqOverride = 4
		return true
	end
		
	if not pl:OnGround() or pl:WaterLevel() >= 3 then
		pl.CalcSeqOverride = 5
	elseif velocity:Length2D() <= 0.5 then
		if wep:IsRoaring() then
			pl.CalcSeqOverride = 16
		else
			pl.CalcIdeal = ACT_IDLE
		end
	else
		pl.CalcIdeal = ACT_RUN
	end

	return true
end

function CLASS:UpdateAnimation(pl, velocity, maxseqgroundspeed)
	pl:FixModelAngles(velocity)

	local seq = pl:GetSequence()
	if seq == 5 or seq == 8 then
		pl:SetPlaybackRate(1)
		return true
	end
	
	if seq == 10 then
		local zvel = pl:GetVelocity().z
		if math.abs(zvel) < 32 then zvel = 0 end
		pl:SetPlaybackRate(math.Clamp(zvel / 160, -1, 1))
		return true
	end

	if seq == 4 then
		if not pl.m_PrevFrameCycle then
			pl.m_PrevFrameCycle = true
			pl:SetCycle(0)
		end

		pl:SetPlaybackRate(0.25)

		return true
	elseif seq == 16 or seq == 1 then
		if not pl.m_PrevFrameCycle then
			pl.m_PrevFrameCycle = true
			pl:SetCycle(0)
		end
	elseif pl.m_PrevFrameCycle then
		pl.m_PrevFrameCycle = nil
	end
end

if not CLIENT then return end

CLASS.HealthBar = surface.GetTextureID("zombiesurvival/healthbar_fastzombie")
CLASS.Icon = "zombiesurvival/killicons/fastzombie"

function CLASS:CreateMove(pl, cmd)
	local wep = pl:GetActiveWeapon()
	if wep:IsValid() and wep.m_ViewAngles and wep:IsPouncing() then
		local maxdiff = FrameTime() * 20
		local mindiff = -maxdiff
		local originalangles = wep.m_ViewAngles
		local viewangles = cmd:GetViewAngles()

		local diff = math.AngleDifference(viewangles.yaw, originalangles.yaw)
		if diff > maxdiff or diff < mindiff then
			viewangles.yaw = math.NormalizeAngle(originalangles.yaw + math.Clamp(diff, mindiff, maxdiff))
		end

		wep.m_ViewAngles = viewangles

		cmd:SetViewAngles(viewangles)
	end
end
