CLASS.Name = "Ghoul"
CLASS.Description = "This zombie has highly toxic flesh. It's slightly weaker than other zombies but makes up for it with its poison attacks. Its claws can debilitate a human for a short time and it will eject poison through its wounds if hit with enough damage."

--CLASS.Wave = 1 / 3
CLASS.Wave = 0
CLASS.Unlocked = true

CLASS.Health = 125
CLASS.Speed = 140

CLASS.Frags = 4

CLASS.SWEP = "weapon_zs_ghoul"

CLASS.Model = Model("models/Zombie/Classic.mdl")

CLASS.VoicePitch = 0.7

function CLASS:PlayPainSound(pl)
	pl:EmitSound("npc/zombie_poison/pz_warn"..math.random(1, 2)..".wav", 75, math.Rand(137, 143))

	return true
end

function CLASS:PlayDeathSound(pl)
	pl:EmitSound("npc/zombie_poison/pz_die2.wav", 75, math.Rand(122, 128))

	return true
end

function CLASS:KnockedDown(pl, status, exists)
	pl:AnimResetGestureSlot(GESTURE_SLOT_ATTACK_AND_RELOAD)
end

function CLASS:CalcMainActivity(pl, velocity)
	--[[local feign = pl.FeignDeath
	if feign and feign:IsValid() then
		pl.CalcSeqOverride = 27
	else]] if velocity:Length2D() <= 0.5 then
		pl.CalcIdeal = ACT_IDLE
	else
		pl.CalcSeqOverride = math.ceil((CurTime() + pl:EntIndex()) * 0.25 % 3) -- This is just so the walk animation being chosen is consistent on the server and client.
	end

	return true
end

--[[function CLASS:ShouldDrawLocalPlayer(pl)
	return IsValid(pl.FeignDeath)
end]]

function CLASS:UpdateAnimation(pl, velocity, maxseqgroundspeed)
	pl:FixModelAngles(velocity)

	--[[local feign = pl.FeignDeath
	if feign and feign:IsValid() then
		if feign:GetState() == 1 then
			pl:SetCycle(1 - math.max(feign:GetStateEndTime() - CurTime(), 0) * 0.5)
		else
			pl:SetCycle(math.max(feign:GetStateEndTime() - CurTime(), 0) * 0.5)
		end
	end]]
end

function CLASS:DoAnimationEvent(pl, event, data)
	if event == PLAYERANIMEVENT_ATTACK_PRIMARY then
		pl:AnimRestartGesture(GESTURE_SLOT_ATTACK_AND_RELOAD, ACT_MELEE_ATTACK1)
		return ACT_INVALID
	end
end

--[[function CLASS:DoesntGiveFear(pl)
	return IsValid(pl.FeignDeath)
end]]

local function CreateFlesh(pl, damage, damagepos, damagedir)
	damage = math.min(damage, 100)

	pl:EmitSound("physics/body/body_medium_break"..math.random(2, 4)..".wav", 74, 125 - damage * 0.50)

	if SERVER then
		local damagepos = pl:LocalToWorld(damagepos)

		for i=1, math.floor(damage / 15) do
			local ent = ents.Create("projectile_poisonflesh")
			if ent:IsValid() then
				local heading = (damagedir + VectorRand() * 0.3):Normalize()
				ent:SetPos(damagepos + heading)
				ent:SetOwner(pl)
				ent:Spawn()
				ent:SetTeamID(TEAM_UNDEAD)
				local phys = ent:GetPhysicsObject()
				if phys:IsValid() then
					phys:Wake()
					phys:SetVelocityInstantaneous(math.min(350, 16 + damage ^ math.Rand(1.1, 1.2)) * heading)
				end
			end
		end
	end
end

function CLASS:ProcessDamage(pl, damage, attacker, inflictor, dmginfo)
	if attacker ~= pl and damage >= 15 and damage < pl:Health() and CurTime() >= (pl.m_NextGhoulEmit or 0) then
		pl.m_NextGhoulEmit = CurTime() + 0.25

		timer.Simple(0, CreateFlesh, pl, damage, pl:WorldToLocal(dmginfo:GetDamagePosition()), dmginfo:GetDamageForce():Normalize() * -1)
	end
end

if SERVER then
	--[[function CLASS:AltUse(pl)
		local feigndeath = pl.FeignDeath
		if feigndeath and feigndeath:IsValid() then
			if CurTime() >= feigndeath:GetStateEndTime() then
				feigndeath:SetState(1)
				feigndeath:SetStateEndTime(CurTime() + 2)
			end
		elseif pl:IsOnGround() and not pl:KeyDown(IN_FORWARD) and not pl:KeyDown(IN_MOVERIGHT) and not pl:KeyDown(IN_MOVELEFT) and not pl:KeyDown(IN_BACK) then
			local wep = pl:GetActiveWeapon()
			if wep:IsValid() and not wep:IsSwinging() and CurTime() > wep:GetNextPrimaryFire() then
				wep:StopMoaning()

				local status = pl:GiveStatus("feigndeath")
				if status and status:IsValid() then
					status:SetStateEndTime(CurTime() + 2)
				end
			end
		end
	end]]

	function CLASS:OnKilled(pl, attacker, inflictor, suicide, headshot, dmginfo, assister)
		local damage = dmginfo:GetDamage()
		if damage >= 10 then
			timer.Simple(0, CreateFlesh, pl, math.max(30, damage * 2), pl:WorldToLocal(dmginfo:GetDamagePosition()), dmginfo:GetDamageForce():Normalize() * -1)
		end
	end

	function CLASS:OnSpawned(pl)
		local status = pl:GiveStatus("overridemodel")
		if status and status:IsValid() then
			status:SetModel("models/player/corpse1.mdl")
		end
	end
end

if CLIENT then
	CLASS.HealthBar = surface.GetTextureID("zombiesurvival/healthbar__human")
	CLASS.Icon = "zombiesurvival/killicons/zombie"
end
