CLASS.Name = "Zombie"
CLASS.Description = "The basic zombie is very durable and has powerful claws. It's hard to keep down, especially if you don't aim for the head."

CLASS.Wave = 0
CLASS.Unlocked = true
CLASS.IsDefault = true
CLASS.Order = 0

CLASS.Health = 200
CLASS.Speed = 140
CLASS.Revives = true

CLASS.Frags = 5

CLASS.SWEP = "weapon_zs_zombie"

CLASS.Model = Model("models/Zombie/Classic.mdl")

CLASS.PainSounds = {"npc/zombie/zombie_pain1.wav", "npc/zombie/zombie_pain2.wav", "npc/zombie/zombie_pain3.wav", "npc/zombie/zombie_pain4.wav", "npc/zombie/zombie_pain5.wav", "npc/zombie/zombie_pain6.wav"} --{"Zombie.Pain"}
CLASS.DeathSounds = {"npc/zombie/zombie_die1.wav", "npc/zombie/zombie_die2.wav", "npc/zombie/zombie_die3.wav"}

CLASS.VoicePitch = 0.65

function CLASS:KnockedDown(pl, status, exists)
	pl:AnimResetGestureSlot(GESTURE_SLOT_ATTACK_AND_RELOAD)
end

function CLASS:ShouldDrawLocalPlayer(pl)
	return IsValid(pl.FeignDeath)
end

function CLASS:CalcMainActivity(pl, velocity)
	local revive = pl.Revive
	if revive and revive:IsValid() then
		if revive:IsRising() then
			pl.CalcSeqOverride = 25 --27
		else
			pl.CalcSeqOverride = 22 --26
		end
	else
		local feign = pl.FeignDeath
		if feign and feign:IsValid() then
			pl.CalcSeqOverride = 27
		elseif velocity:Length2D() <= 0.5 then
			local wep = pl:GetActiveWeapon()
			if wep:IsValid() and wep.IsMoaning and wep:IsMoaning() then
				pl.CalcIdeal = ACT_IDLE_ON_FIRE
			else
				pl.CalcIdeal = ACT_IDLE
			end
		else
			local wep = pl:GetActiveWeapon()
			if wep:IsValid() and wep.IsMoaning and wep:IsMoaning() then
				pl.CalcIdeal = ACT_WALK_ON_FIRE
			else
				pl.CalcSeqOverride = math.ceil((CurTime() + pl:EntIndex()) * 0.25 % 3) -- This is because it keeps using different sequences if you use ACT_WALK.
			end
		end
	end

	return true
end

function CLASS:UpdateAnimation(pl, velocity, maxseqgroundspeed)
	pl:FixModelAngles(velocity)

	local feign = pl.FeignDeath
	if feign and feign:IsValid() then
		if feign:GetState() == 1 then
			pl:SetCycle(1 - math.max(feign:GetStateEndTime() - CurTime(), 0) * 0.666)
		else
			pl:SetCycle(math.max(feign:GetStateEndTime() - CurTime(), 0) * 0.666)
		end
		pl:SetPlaybackRate(0)

		return true
	end

	local len2d = velocity:Length2D()
	if len2d > 0.5 then
		pl:SetPlaybackRate(math.min(len2d / maxseqgroundspeed * 0.75, 3))
	else
		pl:SetPlaybackRate(1)
	end

	return true
end

function CLASS:DoAnimationEvent(pl, event, data)
	if event == PLAYERANIMEVENT_ATTACK_PRIMARY then
		pl:AnimRestartGesture(GESTURE_SLOT_ATTACK_AND_RELOAD, ACT_MELEE_ATTACK1)
		return ACT_INVALID
	end
end

function CLASS:DoesntGiveFear(pl)
	return IsValid(pl.FeignDeath)
end

if SERVER then
	function CLASS:AltUse(pl)
		local feigndeath = pl.FeignDeath
		if feigndeath and feigndeath:IsValid() then
			if CurTime() >= feigndeath:GetStateEndTime() then
				feigndeath:SetState(1)
				feigndeath:SetStateEndTime(CurTime() + 1.5)
			end
		elseif pl:IsOnGround() and not pl:KeyDown(IN_FORWARD) and not pl:KeyDown(IN_MOVERIGHT) and not pl:KeyDown(IN_MOVELEFT) and not pl:KeyDown(IN_BACK) then
			local wep = pl:GetActiveWeapon()
			if wep:IsValid() and not wep:IsSwinging() and CurTime() > wep:GetNextPrimaryFire() then
				wep:StopMoaning()

				local status = pl:GiveStatus("feigndeath")
				if status and status:IsValid() then
					status:SetStateEndTime(CurTime() + 1.5)
				end
			end
		end
	end

	function CLASS:ProcessDamage(pl, damage, attacker, inflictor, dmginfo)
		if attacker ~= pl and damage >= pl:Health() and pl:LastHitGroup() ~= HITGROUP_HEAD and damage < 70 and not inflictor.IsMelee and dmginfo:GetDamageType() ~= DMG_BLAST and dmginfo:GetDamageType() ~= DMG_BURN and (pl.NextZombieRevive or 0) <= CurTime() and pl:LastHitGroup() ~= HITGROUP_LEFTLEG and pl:LastHitGroup() ~= HITGROUP_RIGHTLEG then
			pl.NextZombieRevive = CurTime() + 3

			dmginfo:SetDamage(0)
			pl:SetHealth(10)

			local status = pl:GiveStatus("revive_slump")
			if status then
				status:SetReviveTime(CurTime() + 2.25)
			end

			return true
		--[[elseif damage > 20 then
			pl:RestartGesture(ACT_GESTURE_FLINCH_HEAD)]]
		end
	end

	CLASS.ReviveCallback = function(pl, attacker, dmginfo)
		if not pl.Revive and not dmginfo:GetInflictor().IsMelee and dmginfo:GetDamageType() ~= DMG_BLAST and dmginfo:GetDamageType() ~= DMG_BURN and (pl:LastHitGroup() == HITGROUP_LEFTLEG or pl:LastHitGroup() == HITGROUP_RIGHTLEG) then
			local torso = GAMEMODE.ZombieClasses["Zombie Torso"]
			if torso then
				pl:RemoveStatus("overridemodel", false, true)
				local deathclass = pl.DeathClass or pl:GetZombieClass()
				pl:SetZombieClass(torso.Index)
				pl:DoHulls(torso.Index, TEAM_UNDEAD)
				pl.DeathClass = deathclass

				pl:EmitSound("physics/flesh/flesh_bloody_break.wav", 100, 75)
				local ent = ents.Create("prop_dynamic_override")
				if ent:IsValid() then
					ent:SetModel(Model("models/Zombie/Classic_legs.mdl"))
					ent:SetPos(pl:GetPos())
					ent:SetAngles(pl:GetAngles())
					ent:Spawn()
					ent:Fire("kill", "", 1.5)
				end
				pl:Gib()
				pl.Gibbed = nil

				timer.Simple(0, pl.SecondWind, pl)
				return true
			end
		end

		return false
	end

	function CLASS:OnSecondWind(pl)
		pl:EmitSound("npc/zombie/zombie_voice_idle"..math.random(1, 14)..".wav", 100, 85)
	end
end

if CLIENT then
	CLASS.HealthBar = surface.GetTextureID("zombiesurvival/healthbar_zombiefix")
	CLASS.Icon = "zombiesurvival/killicons/zombie"
end
